/*
	Authorization code smaple for Elektron Data Platform
	
	Prerequisite: Register Your Application
	---------------------------------------
	Your application needs an Application Key with an associated Redirect URI. 
	
	Application Key:  is used in the OAuth 2.0 calls as the client_id parameter. 
	Redirect URI: should handle the requesting of the authorization code and the access token when necessary. 
	After you have created the Application Key, please get the redirect URI registered by sending an email to the account manager.

	To create an application key please visit: https://apps.platform.refinitiv.com/apps/AppkeyGenerator
	Until you have your own Redirect URI, you can use the sample values shown here
*/ 
 
 
// Your application key
const myRDP_ApplicationID = 'D6D73083EAE319C6CDDD74';
// Your registered redirect URI
const myRDP_redirectUri = 'http://localhost:5555/signin-aaa';



"option explicit";

const path = require('path');
const express = require('express');
const serveStatic = require('serve-static');
const ClientOAuth2 = require('client-oauth2');
const request = require('request');


// RDP AAA OAuth 2.0 Configuration
const aaaAuth = new ClientOAuth2({
	// server info
	authorizationUri: 'https://api.refinitiv.com/auth/oauth2/v1/authorize',
	accessTokenUri: 'https://api.refinitiv.com/auth/oauth2/v1/token',
	revokeUri: 'https://api.refinitiv.com/auth/oauth2/v1/revoke',

	// client info
	redirectUri: myRDP_redirectUri,
	clientId: myRDP_ApplicationID,	
	clientSecret: '',

	scopes: ['trapi'],
	state: 'st12ate'
});


const app = express();

// In this simple sample, we do not set up a proper session. Instead user information is saved in a global variable.
// Use a preferred session management solution, e.g. express-session or cookie-session.
var userData = null;

// set the user data (access/refresh tokens)
function setSession(data) {
	userData = data;
}

// retrieve the user data
function getSession() {
	return userData;
}


// use hbs for templating and look in /views for templates
app.set("view engine", "hbs");
app.set("views", path.join(__dirname, "views"));
// avoid direct references to node_modules in the templates...
app.use('/bootstrap', serveStatic(path.join(__dirname, "node_modules/bootstrap/dist")));
app.use('/jquery', serveStatic(path.join(__dirname, "node_modules/jquery/dist")));


// Index page
app.get('/', (req, res) => {
	var user = getSession();
	res.render("index", {accessToken: user && user.accessToken} );
});



// Issue refresh grant
app.get('/refresh', (req, res) => {
	// refresh the current users access token.
	var user = getSession();
	if (user && user.refresh && user.refreshToken) {
		user.refresh()
		.then(function (updatedUser) {
			setSession(updatedUser);
			res.redirect('/');
		})
		.catch(err => {
			res.render("/error", { message: err});
		});
	}
	else {
		// initiate Authorization Code Grant
		var uri = aaaAuth.code.getUri()
		res.redirect(uri)
	}
});





// Code end-point, called by the authorization server to pass code
app.get('/signin-aaa', (req, res) => {
	// get the tokens through backchannel
	aaaAuth.code.getToken(req.originalUrl)
	.then(function (user) {
		setSession(user);
		res.redirect("/");
	})
});



// Manually issue a logout
app.get('/logout', (req, res) => {
	// Logout the current user
	var user = getSession();
	if (user && user.refresh && user.refreshToken) {
		const revokeUri = aaaAuth.options.revokeUri;
		const clientId = aaaAuth.options.clientId;
		const clientSecret = aaaAuth.options.clientSecret;
		const refreshToken = user.refreshToken;

		// our client library (client-oauth2) doesn't support RFC 7009 (OAuth 2.0 Token Revocation)
		// so has to be done manually

		// use client ID and secret as auth header
		const authHeader = Buffer.from(clientId + ':' + clientSecret).toString('base64');

		const options = {
			url: revokeUri,
			method: 'POST',
			headers: {
				Authorization: 'Basic ' + authHeader,
			},
			form: {
				'token': refreshToken,
				'token_type_hint': 'refresh_token',
			},
		};

		request(options, (error, response, body) => {
			if (error || response.statusCode != 200) {
				const message = error ? error : ( body.error_description ? body.error_description : body );
				res.render("error", {
					contentTitle: "Error",
					message: "Error revoking: "+message,
				});
			}
			else {
				user = null;
				setSession(user);
				res.redirect('/');
			}
		});
	}
	else {
		res.redirect('/');
	}
});



const portNumber = 5555;
console.log("Application listening on http://localhost:" + portNumber + "/");
app.listen(portNumber);
